@@ -1,14 +1,17 @@
 #include "playlist.h"
 #include "../../models/tracks/tracksmodel.h"
 
-#include <QRandomGenerator>
+// #include <QRandomGenerator>
 #include <QUrl>
 #include <QDebug>
 #include <MauiKit/Core/utils.h>
 
+#include <random>
+
 Playlist::Playlist(QObject *parent)
     : QObject(parent)
     , m_playMode(static_cast<Playlist::PlayMode>(UTIL::loadSettings("PLAYMODE", "PLAYBACK", 0).toUInt()))
+    ,m_autoResume(UTIL::loadSettings("autoResume", "Settings", -1).toBool())
 {
 }
 
@@ -37,6 +40,11 @@ Playlist::RepeatMode Playlist::repeatMode() const
     return m_repeatMode;
 }
 
+bool Playlist::autoResume() const
+{
+    return m_autoResume;
+}
+
 void Playlist::loadLastPlaylist()
 {
     if (!m_model)
@@ -91,8 +99,6 @@ void Playlist::next()
         return;
     }
 
-    m_previousIndex = m_currentIndex;
-
     switch(m_repeatMode)
     {
 
@@ -122,14 +128,9 @@ void Playlist::next()
     switch(m_playMode)
     {
     case PlayMode::Normal:
-    {
-        setCurrentIndex(m_currentIndex + 1 >= m_model->getCount() ? 0 : m_currentIndex + 1);
-        break;
-    }
-
     case PlayMode::Shuffle:
     {
-        nextShuffle();
+        setCurrentIndex(m_currentIndex + 1 >= m_model->getCount() ? 0 : m_currentIndex + 1);
         break;
     }
     }
@@ -142,20 +143,12 @@ void Playlist::previous()
         return;
     }
 
-    auto previous = m_currentIndex - 1 >= 0 ? m_currentIndex - 1 : m_model->getCount() - 1;
-    m_previousIndex = m_currentIndex;
-    setCurrentIndex(previous);
-}
-
-void Playlist::nextShuffle()
-{
-    if (!m_model)
-    {
+    if(!canGoPrevious())
         return;
-    }
 
-    auto count = m_model->getCount();
-    setCurrentIndex(std::rand() % count);
+    int previous = m_currentIndex - 1 >= 0 ? m_currentIndex - 1 : m_model->getCount() - 1;
+
+    setCurrentIndex(previous);
 }
 
 void Playlist::play(int index)
@@ -182,7 +175,7 @@ void Playlist::save()
     }
 
     QStringList urls;
-    const auto count = std::min(m_model->getCount(), 15);
+    const auto count = m_model->getCount();
 
     for (int i = 0; i < count; i++) {
         auto url = m_model->get(i).value("url").toString();
@@ -231,8 +224,6 @@ void Playlist::setModel(TracksModel *model)
     m_model = model;
 
     connect(m_model, &TracksModel::countChanged, this, &Playlist::canPlayChanged);
-
-    this->loadLastPlaylist();
     emit modelChanged(m_model);
 }
 
@@ -291,9 +282,61 @@ void Playlist::setPlayMode(Playlist::PlayMode playMode)
 
     m_playMode = playMode;
     UTIL::saveSettings("PLAYMODE", m_playMode, "PLAYBACK");
+
+    if (playMode == Playlist::PlayMode::Shuffle)
+    {
+        this->shuffleRange(0, m_model->getCount());
+    }
+
     emit playModeChanged(m_playMode);
 }
 
+void Playlist::shuffleRange(int start, int stop)
+{
+    int len = stop - start;
+
+    std::vector<int> shuffled_offsets = {};
+    shuffled_offsets.reserve(len);
+    for (int i = 0; i < len; i++) {
+        shuffled_offsets.push_back(i);
+    }
+    std::random_device rd;
+    std::mt19937 g{rd()};
+    // This isn't really a great randomness source:
+    // https://stackoverflow.com/questions/45069219/how-to-succinctly-portably-and-thoroughly-seed-the-mt19937-prng
+    // https://stackoverflow.com/questions/18880654/why-do-i-get-the-same-sequence-for-every-run-with-stdrandom-device-with-mingw
+    // https://en.cppreference.com/w/cpp/numeric/random/random_device
+    // But at least in recent versions of most compilers, it should generate a new sequence each time:
+    // https://gcc.gnu.org/bugzilla/show_bug.cgi?id=85494
+    std::shuffle(
+        shuffled_offsets.begin(),
+        shuffled_offsets.end(),
+        g
+    );
+
+    std::vector<QVariantMap> shuffled_tracks = {};
+    int new_index = -1;
+
+    for (int i = 0; i < len; i++) {
+        int remap_i = start + shuffled_offsets[i];
+        shuffled_tracks.push_back(m_model->get(remap_i));
+        if (remap_i == currentIndex())
+        {
+            new_index = start + i;
+        }
+    }
+
+    int i = start;
+    for (auto track: shuffled_tracks) {
+        m_model->update(track, i++);
+    }
+
+    if (new_index != -1)
+    {
+        changeCurrentIndex(new_index);
+    }
+}
+
 void Playlist::move(int from, int to)
 {
     if(!m_model)
@@ -305,7 +348,6 @@ void Playlist::move(int from, int to)
 
     qDebug() << "changing current track index" << from << to << m_currentIndex;
 
-
     if(from == m_currentIndex)
     {
         changeCurrentIndex(to);
@@ -348,3 +390,25 @@ void Playlist::setRepeatMode(Playlist::RepeatMode repeatMode)
     emit repeatModeChanged(m_repeatMode);
 }
 
+void Playlist::setAutoResume(bool autoResume)
+{
+    if (m_autoResume == autoResume)
+        return;
+
+    m_autoResume = autoResume;
+    UTIL::saveSettings("autoResume", m_autoResume, "Settings");
+    emit autoResumeChanged(m_autoResume);
+}
+
+void Playlist::componentComplete()
+{
+    if(m_autoResume)
+    {
+        this->loadLastPlaylist();
+    }
+}
+
+void Playlist::classBegin()
+{
+
+}
