@@ -1,8 +1,10 @@
 .import org.mauikit.filebrowsing 1.3 as FB
 
+.import org.maui.vvave 1.0 as Vvave
+
 function playTrack()
 {
-    player.url = currentTrack.url;
+    player.url = currentTrack.url ? currentTrack.url : "";
     player.play()
 }
 
@@ -10,9 +12,10 @@ function queueTracks(tracks)
 {
     if(tracks && tracks.length > 0)
     {
-        appendTracksAt(tracks, currentTrackIndex+onQueue+1)
-        root.notify("", "Queue", tracks.length + " tracks added put on queue")
-        onQueue++
+        appendTracksAt(tracks, currentTrackIndex+1)
+        // root.notify("", "Queue", tracks.length + " tracks added put on queue")
+        // This freezes the whole UI. It should probably be a toast popup instead.
+        // Something similar AbstractApplicationWindow::showPassiveNotification() from Kirigami would be less flow-breaking.
     }
 }
 
@@ -123,6 +126,12 @@ function appendAllModel(model)
 {
     mainPlaylist.listModel.list.copy(model)
     mainPlaylist.listView.positionViewAtEnd()
+
+    if (model.count > 1 && root.playlistManager.playMode == Vvave.Playlist.Shuffle)
+        root.playlistManager.shuffleRange(
+            mainPlaylist.listView.count - model.count,
+            mainPlaylist.listView.count
+        )
 }
 
 function playAllModel(model)
